
require 'racc/parser'
#
# confp.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'est/confs'

module EST

class ConfigFileParser < Racc::Parser

  class << self
    def parse( fn )
      new.parse( fn )
    end
  end

  def initialize( dflag = false )
    @yydebug = dflag
  end

  def parse( fn )
    unless File.exist? fn then
      raise ArgumentError, "config file #{fn} not exist"
    end
    str = nil
    File.open( fn ) {|f| str = f.read }
    @scanner = ConfigFileScanner.new( str )

    do_parse
  end

  private

  def next_token
    @scanner.scan
  end

  def on_error( et, ev, _values )
    raise ParseError, <<S

parse error in line #{@scanner.lineno}
S
  end


  def set_value0( hash )
    h = @hstack[-1]
    h.update hash
  end
    
  def set_value1( key, val )
    h = @hstack[-1]
    h[ key ] = val
  end

  def set_value2( key1, key2, val )
    h = @hstack[-1]
    if tmp = h[ key1 ] then
      tmp[ key2 ] = val
    else
      h[ key1 ] = { key2 => val }
    end
  end

##### racc 0.11.0 generates ###

Racc_reduce_table = [
0, 0, :racc_error,
0, 14, :_reduce_1,
2, 13, :_reduce_2,
1, 12, :_reduce_none,
3, 12, :_reduce_none,
0, 15, :_reduce_none,
2, 15, :_reduce_6,
3, 15, :_reduce_7,
2, 15, :_reduce_8,
3, 15, :_reduce_9,
4, 15, :_reduce_10,
1, 16, :_reduce_none,
1, 16, :_reduce_none,
1, 19, :_reduce_none,
1, 19, :_reduce_none,
0, 22, :_reduce_15,
4, 20, :_reduce_16,
3, 21, :_reduce_17,
0, 23, :_reduce_18,
3, 23, :_reduce_19,
0, 24, :_reduce_none,
1, 24, :_reduce_none,
1, 24, :_reduce_none,
1, 17, :_reduce_none,
2, 17, :_reduce_24,
1, 18, :_reduce_none,
1, 18, :_reduce_none ]

Racc_reduce_n = 27

Racc_shift_n = 36

Racc_action_table = [
    -1,    -1,     0,     3,    -1,   -27,     5,     5,     4,     6,
     3,     8,    -1,    -5,     0,    10,    -1,   -27,     7,    11,
     9,    12,    -1,   -27,    -1,   -26,     5,    16,     6,    17,
     3,    19,    -1,   -25,    -1,    -3,     4,    20,    -1,   -27,
     2,    21,     0,    -2,    -1,   -27,    -1,    36,    -1,   -15,
    -1,   -18,    -1,   -14,    -1,    -8,    -1,   -13,     7,    11,
     9,    12,     3,    25,    -1,   -27,    -1,   -11,     5,     5,
     4,    28,    -1,   -27,    -1,   -12,    -1,    -6,     5,     5,
     4,     6,     3,     8,    -1,    -5,     5,     5,     4,     6,
     3,     8,    -1,    -5,     2,    31,    10,    32,    -1,   -27,
    -1,    -9,     4,    33,    -1,   -27,    -1,   -23,     5,     5,
     4,    28,    -1,    -7,    -1,   -25,    -1,    -4,     2,    21,
     8,    35,    -1,   -27,     7,    11,     5,     5,     9,    12,
     4,    28,    -1,   -20,    -1,   -17,    -1,   -10,    -1,   -24,
    -1,   -16,     5,     5,     4,    28,    -1,   -21,    -1,   -22,
    -1,   -19 ]

Racc_action_table_ptr = [
     0,     2,     6,    14,    18,    24,    26,    34,    36,    40,
    46,    48,    50,    52,    54,    56,    58,    66,    68,    74,
    76,    78,    86,    94,   100,   102,   106,   108,   114,   116,
   118,   124,   134,   136,   138,   140,   142,   148,   150 ]


Racc_goto_table = [
    13,     1,    14,     2,    18,     4,    15,     7,    12,     9,
    21,    13,    19,    14,    20,    15,    16,    18,    22,    22,
    23,    23,    21,    13,    19,    24,    20,    15,    18,    26,
    17,    27,    18,     4,    15,    29,    18,     4,    15,     7,
    12,    30,    18,    34,    18,    26,    17,    36,    19,    37,
    21,    13,    24,    38,    20,    15,    18,    34,    -1,    -1 ]

Racc_goto_table_ptr = [
     0,    -1,     4,    -1,    10,    -1,    16,    -1,    -1,    -1,
    -1,    18,    20,    -1,    -1,    -1,    22,    -1,    28,    -1,
    -1,    32,    36,    -1,    -1,    -1,    -1,    42,    -1,    -1,
    -1,    44,    -1,    -1,    -1,    -1,    56,    -1,    -1 ]


Racc_token_table = {
 false => 0,
 Object.new => 1,
 :EOL => 2,
 "=" => 3,
 :STRING => 4,
 :QUOTED => 5,
 ":" => 6,
 "{" => 7,
 "}" => 8,
 "[" => 9,
 "]" => 10 }

Racc_debug_parser = false

##### racc system variables end #####

 # reduce 0 omitted

 module_eval( '', 'rr.y', 12 ) {
  def _reduce_1( val, _values, result )
                    @hstack = [ {} ]
   result
  end
 }

 module_eval( '', 'rr.y', 16 ) {
  def _reduce_2( val, _values, result )
                    result = @hstack[0]
   result
  end
 }

 # reduce 3 omitted

 # reduce 4 omitted

 # reduce 5 omitted

 module_eval( '', 'rr.y', 27 ) {
  def _reduce_6( val, _values, result )
                    hash = ConfigFileParser.parse(
                      CORE.search_file( val[1], 'config' ) )
                    set_value0 hash
   result
  end
 }

 module_eval( '', 'rr.y', 33 ) {
  def _reduce_7( val, _values, result )
                    set_value1 result, val[2]
   result
  end
 }

 module_eval( '', 'rr.y', 37 ) {
  def _reduce_8( val, _values, result )
                    set_value1 result, val[1]
   result
  end
 }

 module_eval( '', 'rr.y', 41 ) {
  def _reduce_9( val, _values, result )
                    set_value2 result, val[1], val[2]
   result
  end
 }

 module_eval( '', 'rr.y', 45 ) {
  def _reduce_10( val, _values, result )
                    hash = ConfigFileParser.parse(
                      CORE.search_file( val[3], 'config' ) )
                    set_value2 result, val[1], hash
   result
  end
 }

 # reduce 11 omitted

 # reduce 12 omitted

 # reduce 13 omitted

 # reduce 14 omitted

 module_eval( '', 'rr.y', 59 ) {
  def _reduce_15( val, _values, result )
                    @hstack.push Hash.new
   result
  end
 }

 module_eval( '', 'rr.y', 63 ) {
  def _reduce_16( val, _values, result )
                    result = @hstack.pop
   result
  end
 }

 module_eval( '', 'rr.y', 69 ) {
  def _reduce_17( val, _values, result )
                    result = val[1]
   result
  end
 }

 module_eval( '', 'rr.y', 75 ) {
  def _reduce_18( val, _values, result )
                    result = []
   result
  end
 }

 module_eval( '', 'rr.y', 79 ) {
  def _reduce_19( val, _values, result )
                    result.push val[2] if val[2]
   result
  end
 }

 # reduce 20 omitted

 # reduce 21 omitted

 # reduce 22 omitted

 # reduce 23 omitted

 module_eval( '', 'rr.y', 91 ) {
  def _reduce_24( val, _values, result )
                    result = [result] unless Array === result
                    result.push val[1]
   result
  end
 }

 # reduce 25 omitted

 # reduce 26 omitted

end

end   # module EST
