#
# estcore.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'singleton'

require 'amstd/must'
require 'amstd/bug'
require 'amstd/futils'


module EST

  Version = '0.0.6'

  def EST.create
    const_set :CORE, Core.instance
  end


  class CCore

    Export = %w(
      [] mailbox sender popper composer getter refiler
      notice warn shutdown
      mvmail rmmail mkmail refile
      change_profile profile= )

    include Singleton


    def initialize
      @corerc = {}
      initialize_core_rc()

      @mailbox  = Mailset.new
      @sender   = SenderManager.new
      @popper   = PopperManager.new
      @composer = ComposerManager.new
      @getter   = GetterManager.new
      @refiler  = RefilerManager.new
      @profile  = ProfileManager.new

      @addrbook = nil
    end


    attr :mailbox
    attr :sender
    attr :popper
    attr :composer
    attr :getter
    attr :refiler
    attr :profile


    private


    require 'est/estconf'   # def initialize_core_rc()

    def construct
      ensure_config
      first_profile = RcReader.read( search_file( 'core', 'config' ) )
      first_profile
    end

    def ensure_config
      confdir = @corerc['confdir']
      unless File.exist? confdir then
        # FileUtils.mkdir_p confdir
bug!
      end
      unless File.directory? confdir then
        raise ArgumentError, "config directory '#{confdir}' is not directory"
      end
      cf = File.join( confdir, 'core' )
      unless File.exist? cf then
bug! cf
        FileUtils.touch cf
      else
        unless File.file? cf then
bug! cf
          raise ArgumentError, "config file '#{cf}'is not file"
        end
      end
    end


    public


    def shutdown
puts 'EST::CORE.shutdown'
      @mailbox.shutdown if @mailbox
puts 'done'
    end


    def notice( str )
      $stderr.puts str
    end

    def warn( str )
      $stderr.puts str
    end
    
    def []( key )
      @corerc[key]
    end


    def search_file( fname, addpath = '' )
      %w( usrdir libdir ).each do |loc|
        path = File.join( @corerc[ loc ], addpath, fname )
        return path if File.exist? path
      end

      raise ArgumentError,
        "could not find file '#{File.join addpath, fname}'"
    end


    def mvmail( from, to, idx )
      return unless from = to_mbox_from( from )
      return unless to   = to_mbox_to( to )
      return if from == to
      return unless from[idx]

      to.moveof from, idx
    end

    def rmmail( mb, idx )
      return unless mbox = to_mbox_from( mb )
      mbox.remove idx
    end

    def mkmail( mb = nil, str = nil )
      if mb then
        mbox = to_mbox_to( mb )
        return unless mbox
      else
        mbox = mailbox.draft
      end

      port = mbox.new
      port.wopen do |wp|
        if iterator? then
          yield wp
        elsif str then
          wp.write str
        else
          raise ArgumentError, "mkmail: no argument"
        end
      end
      mbox.add port

      return port
    end

    def to_mbox_to( to )
      case to
      when Mailbox then to
      when String  then @mailbox[ to ]
      else
        to.must Mailbox, String
      end
    end

    def to_mbox_from( from )
      case from
      when Mailbox then from
      when String  then @mailbox[ from ]
      else
        to.must Mailbox, String
      end
    end


    def refile
      unless @refile then
        CORE.warn "no refile rule"
        return
      end
      @refile.refile
    end

  end

  class Core < CCore
  end


end   # module EST
